# -*- coding: utf-8 -*-

"""
Request and document
"""

import datetime

from django.contrib.auth.models import User
from django.db import models
from django import forms
from django.utils.translation import ugettext_lazy as _


class DocumentUsage(models.Model):

    class Meta:
        app_label = 'base'
        verbose_name = _(u"Usage d'un document")
        verbose_name_plural = _(u"Usages d'un document")

    def __unicode__(self):
        return self.name

    name = models.CharField(max_length=64, verbose_name=_(u'Usage'))
    description = models.TextField(verbose_name=_(u'Description'), blank=True)
    url = models.URLField(max_length=200, verbose_name=_(u'URL'), blank=True)
    comments = models.TextField(verbose_name=_(u'Remarques et commentaires'), blank=True)


class DocumentLicence(models.Model):

    class Meta:
        app_label = 'base'
        verbose_name = _(u"Diffusion d'un document")
        verbose_name_plural = _(u"Diffusions d'un document")

    def __unicode__(self):
        return self.name

    name = models.CharField(max_length=64, verbose_name=_(u'Droit de diffusion'))
    description = models.TextField(verbose_name=_(u'Description'), blank=True)
    url = models.URLField(max_length=200, verbose_name=_(u'URL'), blank=True)
    comments = models.TextField(verbose_name=_(u'Remarques et commentaires'), blank=True)


class Document(models.Model):

    class Meta:
        app_label = 'base'
        verbose_name = _(u'Document');
        verbose_name_plural = _(u'Documents')

    def __unicode__(self):
        return self.name

    user = models.ForeignKey(User, verbose_name=_(u'Utilisateur ayant déposé le fichier'))
    name = models.CharField(max_length=128, verbose_name=_(u'Nom du document'))
    uploadfile = models.FileField(upload_to='upload', verbose_name=_(u'Fichier'), blank=True)
    nb_pages = models.IntegerField(verbose_name=_(u'Nombre de pages'))
    usage = models.ForeignKey(DocumentUsage, verbose_name=_(u'Usage du document'))
    licence = models.ForeignKey(DocumentLicence, verbose_name=_(u'Droits de diffusion'))
    copyright = models.BooleanField(verbose_name=_(u"Déclaration de présence d'extraits protégés"))
    comments = models.TextField(verbose_name=_(u'Remarques et commentaires'), blank=True)
    creation_date = models.DateField(verbose_name=_(u'Date de dépôt du document'), default=datetime.date.today, editable=False)
    modification_date = models.DateField(verbose_name=_(u'Date de modification'), editable=False)

    def save(self):
        self.modification_date = datetime.date.today()
        super(Document, self).save()


    def is_paper(self):
        return uploadfile is None


class RequestStatus(models.Model):

    class Meta:
        app_label = 'base'
        verbose_name = _(u"Statut d'une demande")
        verbose_name_plural = _(u"Statuts d'une demande")

    def __unicode__(self):
        return self.name

    id = models.CharField(max_length=32, primary_key=True, verbose_name=_(u'Identifiant'))
    name = models.CharField(max_length=64, verbose_name=_(u'Nom du statut'))


class DeliveryPlace(models.Model):

    class Meta:
        app_label = 'base'
        verbose_name = _(u"Lieu de livraison")
        verbose_name_plural = _(u"Lieux de livraison")

    def __unicode__(self):
        return self.name

    name = models.CharField(max_length=64, verbose_name=_(u'Lieu de livraison'))
    description = models.TextField(verbose_name=_(u'Description'), blank=True)
    url = models.URLField(max_length=200, verbose_name=_(u'URL'), blank=True)
    comments = models.TextField(verbose_name=_(u'Remarques et commentaires'), blank=True)


class Request(models.Model):

    class Meta:
        app_label = 'base'
        verbose_name = _(u'Demande de reprographie');
        verbose_name_plural = _(u'Demandes de reprographie')

    user = models.ForeignKey(User, verbose_name=_(u'Demandeur'))
    sponsor = models.CharField(max_length=128, verbose_name=_(u'Nom du commanditaire'), blank=True)
    document = models.ForeignKey(Document, verbose_name=_(u'Document lié à cette demande'))
    entity = models.ForeignKey('Entity', verbose_name=_(u'Entité administrative'), on_delete=models.DO_NOTHING)
    copies = models.IntegerField(verbose_name=_(u'Nombre de copies'))
    status = models.ForeignKey('RequestStatus', verbose_name=_(u'Statut actuel de la demande'))
    base_profile = models.ForeignKey('Profile', verbose_name=_('Profil de reprographie de base'))
    choices = models.ManyToManyField('ProfileOptionChoice', verbose_name=_(u'Tous les choix de reprographie'))
    details = models.TextField(verbose_name=_(u'Prestations complétementaires'), blank=True)
    delivery_date = models.DateField(verbose_name=_(u'Date de livraison souhaitée'))
    delivery_place = models.ForeignKey(DeliveryPlace, verbose_name=_(u'Lieu de livraison'))
    comments = models.TextField(verbose_name=_(u'Remarques et commentaires'), blank=True)
    creation_date = models.DateField(verbose_name=_(u'Date de création'), default=datetime.date.today)
    modification_date = models.DateField(verbose_name=_(u'Date de modification'), editable=False)

    def optimize_profile(self):
        '''
        set base_profile and choices to get the lower price
        '''
        pass # TODO ...

    def save(self, *args, **kwargs):
        self.optimize_profile()
        self.modification_date = datetime.date.today()
        super(Request, self).save(*args, **kwargs)


class History(models.Model):

    class Meta:
        app_label = 'base'
        verbose_name = _(u"Historique")
        verbose_name_plural = _(u"Historiques des demandes")

    def __unicode__(self):
        return self.action

    date = models.DateField()
    request = models.ForeignKey(Request, verbose_name=_(u'Demande'))
    user = models.ForeignKey(User, verbose_name=_(u'Utilisateur'), blank=True, null=True)
    action = models.CharField(max_length=128, verbose_name=_(u"Nature de l'action"), blank=True)
    old_status = models.ForeignKey("RequestStatus",
            verbose_name=_(u"Etat de la demande avant l'action"), related_name='+', blank=True, null=True)
    new_status = models.ForeignKey("RequestStatus",
            verbose_name=_(u"Nouvel état de la demande après l'action"), related_name='+')
    description = models.TextField(verbose_name=_(u"Détails éventuels sur l'action"), blank=True)

