# -*- encoding: utf-8 -*-

from functools import wraps

import django.core.cache

def tuplify(iterable):
    '''Convert all iterable to tuple.

       Use it when you want to index a dictionary with an iterable.
    '''
    if hasattr(iterable, '__iter__'):
        return tuple(tuplify(x) for x in iterable)
    else:
        return iterable

class CachingDecorator(object):
    '''Mixin class for building caching decorators'''
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)

    def cache(self, args, kwargs):
        return None

    def key(self, args, kwargs):
        return tuplify(tuple(args) + tuple(sorted(kwargs.iteritems())))

    def get(self, cache, key, default=None):
        return default

    def set(self, cache, key, value):
        pass

    def __call__(self, function):
        function_name = '{0}.{1}_cache'.format(function.__module__,
                function.__name__)
        @wraps(function)
        def f(*args, **kwargs):
            cache = self.cache(args, kwargs)
            key = (function_name, self.key(args, kwargs))
            print 'key is', key
            result = self.get(cache, key)
            if result:
                return result
            result = function(*args, **kwargs)
            self.set(cache, key, result)
            return result
        return f

class CacheToSession(CachingDecorator):
    def cache(self, args, kwargs):
        return getattr(kwargs.get('request', None), 'session', None)

    def get(self, cache, key, default=None):
        if cache is not None:
            return cache.get(key, default)
        return default

    def set(self, cache, key, value):
        if cache is not None:
            cache[key] = value

'''Decorator adding caching to the request.session to any function'''
cache_to_session = CacheToSession()

class CacheToDjangoCache(CachingDecorator):
    '''Add cache using the Django cache framework to a function

       Parameters:
       cache_name - use a specific named cache backend
       timeout - cache values this amount of time
    '''
    def __init__(self, cache_name=None, timeout=None):
        self.cache_name = cache_name
        self.timeout = timeout

    def cache(self, args, kwargs):
        if self.cache_name:
            return django.core.cache.get_cache(self.cache_name)
        else:
            return django.core.cache.cache

    def get(self, cache, key, default=None):
        return cache.get(key, default)

    def set(self, cache, key, value):
        cache.set(key, value, timeout=self.timeout)

'''Decorator adding caching to the default Django cache backend'''
cache_to_django = CacheToDjangoCache

def slice_collection(iterable, length):
    '''Cut a slicable container into sample of maximum size equals to length'''
    if hasattr(iterable, '__getslice__'):
        i = 0
        while True:
            l = iterable[i:i+length]
            yield l
            if len(l) < length:
                break
            i += length
    else:
        l = []
        for x in iterable:
            l.append(x)
            if len(l) == length:
                yield l
                l = []
        if l:
            yield l
